@@ -167,14 +167,14 @@ void mwait_idle_with_hints(unsigned long ax, unsigned long cx)
 		if (this_cpu_has(X86_FEATURE_CLFLUSH_MONITOR))
 			clflush((void *)&current_thread_info()->flags);
 
-		restricted_branch_speculation_off();
+		restricted_branch_speculation_end();
 
 		__monitor((void *)&current_thread_info()->flags, 0, 0);
 		smp_mb();
 		if (!need_resched())
 			__mwait(ax, cx);
 
-		restricted_branch_speculation_on();
+		restricted_branch_speculation_start();
 	}
 }
 
