@@ -836,7 +836,7 @@ static void init_amd(struct cpuinfo_x86 *c)
 		case 0x10:
 		case 0x12:
 		case 0x16:
-			pr_info_once("AMD: Disabling Indirect Branch Predictor\n");
+			pr_info_once("Disabling Indirect Branch Predictor Support\n");
 			rdmsrl(MSR_F15H_IC_CFG, val);
 			val |= MSR_F15H_IC_CFG_DIS_IND;
 			wrmsrl(MSR_F15H_IC_CFG, val);
