@@ -200,34 +200,42 @@ static int proc_dostring_coredump(struct ctl_table *table, int write,
 
 #ifdef CONFIG_X86
 /* Mutex to serialize IBPB and IBRS runtime control changes */
-DEFINE_MUTEX(spec_ctrl_mutex);
+DEFINE_MUTEX(ubuntu_spec_ctrl_mutex);
 
 unsigned int ibpb_enabled = 0;
-EXPORT_SYMBOL(ibpb_enabled);
+EXPORT_SYMBOL(ibpb_enabled);   /* Required in some modules */
 
 static unsigned int __ibpb_enabled = 0;   /* procfs shadow variable */
 
-static void set_ibpb_enabled(unsigned int val)
+int set_ibpb_enabled(unsigned int val)
 {
-	mutex_lock(&spec_ctrl_mutex);
+	int error = 0;
+	unsigned int prev = ibpb_enabled;
 
-	/* Only enable IBPB if the CPU supports it */
-	if (val && boot_cpu_has(X86_FEATURE_USE_IBPB))
-		ibpb_enabled = 1;
-	else
+	mutex_lock(&ubuntu_spec_ctrl_mutex);
+
+	/* Only enable/disable IBPB if the CPU supports it */
+	if (boot_cpu_has(X86_FEATURE_USE_IBPB)) {
+		ibpb_enabled = val;
+		if (ibpb_enabled != prev)
+			pr_info("Spectre V2 : %s "
+				"Indirect Branch Prediction Barrier\n",
+				ibpb_enabled ? "Enabling" : "Disabling");
+	} else {
 		ibpb_enabled = 0;
+		if (val) {
+			/* IBPB is not supported but we try to turn it on */
+			error = -EINVAL;
+		}
+	}
 
 	/* Update the shadow variable */
 	__ibpb_enabled = ibpb_enabled;
 
-	mutex_unlock(&spec_ctrl_mutex);
-}
+	mutex_unlock(&ubuntu_spec_ctrl_mutex);
 
-inline void ibpb_enable(void)
-{
-	set_ibpb_enabled(1);
+	return error;
 }
-EXPORT_SYMBOL(ibpb_enable);
 
 static int ibpb_enabled_handler(struct ctl_table *table, int write,
 				void __user *buffer, size_t *lenp,
@@ -239,24 +247,31 @@ static int ibpb_enabled_handler(struct ctl_table *table, int write,
 	if (error)
 		return error;
 
-	set_ibpb_enabled(__ibpb_enabled);
-	return 0;
+	return set_ibpb_enabled(__ibpb_enabled);
 }
 
 unsigned int ibrs_enabled = 0;
-EXPORT_SYMBOL(ibrs_enabled);
+EXPORT_SYMBOL(ibrs_enabled);   /* Required in some modules */
 
 static unsigned int __ibrs_enabled = 0;   /* procfs shadow variable */
 
-static void set_ibrs_enabled(unsigned int val)
+int set_ibrs_enabled(unsigned int val)
 {
+	int error = 0;
 	unsigned int cpu;
+	unsigned int prev = ibrs_enabled;
 
-	mutex_lock(&spec_ctrl_mutex);
+	mutex_lock(&ubuntu_spec_ctrl_mutex);
 
 	/* Only enable/disable IBRS if the CPU supports it */
 	if (boot_cpu_has(X86_FEATURE_USE_IBRS_FW)) {
 		ibrs_enabled = val;
+		if (ibrs_enabled != prev)
+			pr_info("Spectre V2 : %s "
+				"Indirect Branch Restricted Speculation%s\n",
+				ibrs_enabled ? "Enabling" : "Disabling",
+				ibrs_enabled == 2 ? " (user space)" : "");
+
 		if (ibrs_enabled == 0) {
 			/* Always disable IBRS */
 			u64 val = x86_spec_ctrl_base;
@@ -272,33 +287,31 @@ static void set_ibrs_enabled(unsigned int val)
 		}
 	} else {
 		ibrs_enabled = 0;
+		if (val) {
+			/* IBRS is not supported but we try to turn it on */
+			error = -EINVAL;
+		}
 	}
 
 	/* Update the shadow variable */
 	__ibrs_enabled = ibrs_enabled;
 
-	mutex_unlock(&spec_ctrl_mutex);
-}
+	mutex_unlock(&ubuntu_spec_ctrl_mutex);
 
-inline void ibrs_enable(void)
-{
-	set_ibrs_enabled(1);
+	return error;
 }
-EXPORT_SYMBOL(ibrs_enable);
 
 static int ibrs_enabled_handler(struct ctl_table *table, int write,
 				void __user *buffer, size_t *lenp,
 				loff_t *ppos)
 {
 	int error;
-	unsigned int cpu;
 
 	error = proc_dointvec_minmax(table, write, buffer, lenp, ppos);
 	if (error)
 		return error;
 
-	set_ibrs_enabled(__ibrs_enabled);
-	return 0;
+	return set_ibrs_enabled(__ibrs_enabled);
 }
 #endif
 
